<?php
namespace Drupal\logout_after_password_change\EventSubscriber;

use Drupal\user\UserDataInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

class OneTimeLogoutSubscriber implements EventSubscriberInterface
{
  /**
   * If the user is logged in by password reset, the user can visit just entity.user.edit_form
   */
  public function checkAccess(): void
  {
    $current_route = \Drupal::routeMatch();
    $route_name = $current_route->getRouteName();
    if($route_name !== 'entity.user.edit_form') {
      $uid = \Drupal::currentUser()->id();
      /** @var UserDataInterface $userData */
      $userData = \Drupal::service('user.data');
      if ($userData->get('logout_after_password_change', $uid,'is_password_reset')) {
        user_logout();
        \Drupal::messenger()->addMessage(t('You have been logged out.'));
      }
    }
  }

  /**
   * If the user is logged out by changing the password, the message is displayed
   */
  public function checkMessage(RequestEvent $event): void
  {
    $current_route = \Drupal::routeMatch();
    $route_name = $current_route->getRouteName();

    if($route_name == 'user.login') {
      if($event->getRequest()->query->get('logout') == 1) {
        \Drupal::messenger()->addMessage(t('Password was changed. Please login again.'));
      }
    }
  }

  public static function getSubscribedEvents(): array
  {
    $events[KernelEvents::REQUEST][] = ['checkAccess', 'checkMessage'];
    return $events;
  }
}
